
# q-expansion of newform 750.2.a.c, downloaded from the LMFDB on 17 November 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 750
weight = 2
poly_data = [-1, -1, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[-1, 0], [1, 0], [0, 0], [0, 2], [2, 1], [1, -3], [-3, 3], [6, 0], [0, -5], [5, -7], [4, 3], [6, -3], [-2, 2], [2, 3], [-9, 3], [-2, 2], [7, 1], [6, -12], [3, 1], [-4, 8], [12, 0], [-3, -3], [2, -10], [-8, 2], [12, -10], [2, 7], [-4, 0], [-2, 10], [-6, 12], [8, -7], [-6, 12], [8, -16], [4, -1], [-20, 2], [-18, 4], [-16, 3], [-6, -9], [11, -15], [-20, 5], [-8, 2], [0, -12], [0, -12], [6, 0], [-6, 18], [-4, -4], [-15, 1], [-4, -12], [-18, 6], [-14, 4], [2, 6], [3, -7], [-18, 14], [-3, -5], [11, 5], [13, -17], [-3, 21], [22, -1], [-3, 9], [2, -20], [20, -4], [-18, 5], [-2, 22], [-30, 5], [-2, 4], [26, -8], [6, -6], [-14, 12], [-8, 8], [12, -20], [0, 0], [8, 11], [14, -16], [14, 6], [-21, 19], [0, 2], [8, -16], [9, -15], [30, -4], [-8, 4], [8, -9], [-4, 8], [12, -4], [-18, 6], [6, 10], [24, -7], [24, -14], [-2, -8], [-18, 8], [-3, 21], [-6, -6], [6, 6], [-2, -14], [-4, 22], [2, -1], [12, -2], [-8, 4], [-2, -8], [-2, 10], [15, 5], [8, -6], [-11, -9], [-2, -20], [-4, 14], [-18, -8], [0, -20], [14, 16], [10, -14], [28, -11], [-22, 16], [24, -27], [-40, 12], [18, -32], [-14, 28], [-44, 6], [-7, 13], [-22, 14], [-26, 9], [12, -1], [0, 16], [15, 21], [2, -6], [30, -28], [30, -2], [12, 2], [28, 2], [-2, -23], [-18, -18], [-6, 2], [-16, -14], [-18, 27], [0, -14], [19, -1], [-36, 12], [10, 24], [-24, 14], [-5, 29], [4, -4], [33, -9], [-32, 4], [-32, -4], [-34, 0], [22, -23], [-4, 18], [-18, 0], [28, -10], [0, 18], [-26, 5], [8, -7], [-12, 6], [-3, 9], [6, -33], [-10, -8], [34, -21], [52, -4], [25, 3], [40, 4], [12, -16], [-6, 30], [-20, 12], [-7, 25], [-38, 28], [-14, -20], [18, -34], [49, -5], [-24, 49], [-12, 21], [8, 27], [-29, 15]]
